@@ -21,6 +21,12 @@
 #include <math.h>
 #include <string.h>
 
+#if ENABLE_FONT_CONFIG
+# include <fontconfig/fontconfig.h>
+# include <pango/pangocairo.h>
+# include <pango/pangofc-fontmap.h>
+#endif
+
 #include "gtksourcemap.h"
 #include "gtksourcebuffer.h"
 #include "gtksourcecompletion.h"
@@ -226,6 +232,44 @@ G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceMap, gtk_source_map, GTK_SOURCE_TYPE_VIEW)
 
 static GParamSpec *properties[N_PROPERTIES];
 
+#if ENABLE_FONT_CONFIG
+static FcConfig *map_font_config;
+
+static void
+load_override_font (GtkSourceMap *map)
+{
+	PangoFontDescription *font_desc;
+	PangoFontMap *font_map;
+
+	if (g_once_init_enter (&map_font_config))
+	{
+		const gchar *font_path = PACKAGE_DATADIR"/fonts/BuilderBlocks.ttf";
+		FcConfig *config = FcInitLoadConfigAndFonts ();
+
+		if (!g_file_test (font_path, G_FILE_TEST_IS_REGULAR))
+			g_debug ("\"%s\" is missing or inaccessible", font_path);
+
+		FcConfigAppFontAddFile (config, (const FcChar8 *)font_path);
+
+		g_once_init_leave (&map_font_config, config);
+	}
+
+	font_map = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
+	pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (font_map), map_font_config);
+	gtk_widget_set_font_map (GTK_WIDGET (map), font_map);
+	font_desc = pango_font_description_from_string ("BuilderBlocks");
+
+	g_assert (map_font_config != NULL);
+	g_assert (font_map != NULL);
+	g_assert (font_desc != NULL);
+
+	g_object_set (map, "font-desc", font_desc, NULL);
+
+	pango_font_description_free (font_desc);
+	g_object_unref (font_map);
+}
+#endif
+
 static void
 get_slider_position (GtkSourceMap *map,
                      int           width,
@@ -272,49 +316,19 @@ get_slider_position (GtkSourceMap *map,
 	slider_area->y -= us_visible_rect.y;
 }
 
-static void
-gtk_source_map_allocate_slider (GtkSourceMap *map)
-{
-	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
-	GdkRectangle area;
-	int width, height;
-	int min, nat;
-
-	g_assert (GTK_SOURCE_IS_MAP (map));
-
-	width = gtk_widget_get_width (GTK_WIDGET (map));
-	height = gtk_widget_get_height (GTK_WIDGET (map));
-
-	if (width == 0 || height == 0)
-		return;
-
-	get_slider_position (map, width, height, &area);
-	gtk_widget_measure (GTK_WIDGET (priv->slider),
-	                    GTK_ORIENTATION_VERTICAL,
-	                    width, &min, &nat, NULL, NULL);
-	area.height = MAX (nat, area.height);
-	gtk_widget_size_allocate (GTK_WIDGET (priv->slider), &area, -1);
-}
-
 static void
 gtk_source_map_rebuild_css (GtkSourceMap *map)
 {
 	GtkSourceMapPrivate *priv;
 	GtkSourceStyleScheme *style_scheme;
 	GtkSourceStyle *style = NULL;
-	GtkSourceStyle *text = NULL;
 	GtkTextBuffer *buffer;
-	GdkRGBA real_bg;
 	const char *color;
 	gboolean use_fg = FALSE;
 	GString *gstr;
 	char *background = NULL;
 	char *foreground = NULL;
 
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	GtkStyleContext *style_context;
-	G_GNUC_END_IGNORE_DEPRECATIONS
-
 	priv = gtk_source_map_get_instance_private (map);
 
 	if (priv->view == NULL)
@@ -362,29 +376,8 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->view));
 	style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (buffer));
 
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	style_context = gtk_widget_get_style_context (GTK_WIDGET (map));
-	if (!gtk_style_context_lookup_color (style_context, "view_bg_color", &real_bg))
-		memset (&real_bg, 0, sizeof (real_bg));
-	G_GNUC_END_IGNORE_DEPRECATIONS
-
 	if (style_scheme != NULL)
 	{
-		if ((text = gtk_source_style_scheme_get_style (style_scheme, "text")))
-		{
-			char *str;
-
-			g_object_get (text,
-				      "background", &str,
-				      NULL);
-
-			if (str != NULL)
-			{
-				gdk_rgba_parse (&real_bg, str);
-				g_free (str);
-			}
-		}
-
 		if (!(style = gtk_source_style_scheme_get_style (style_scheme, "map-overlay")) &&
 		    !(style = gtk_source_style_scheme_get_style (style_scheme, "selection")))
 		{
@@ -468,40 +461,18 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 
 	if (color != NULL)
 	{
-		GdkRGBA to_mix;
-		GdkRGBA normal;
-		GdkRGBA hover;
-		GdkRGBA active;
-		char *normal_str;
-		char *hover_str;
-		char *active_str;
-
-		gdk_rgba_parse (&to_mix, color);
-
-		premix_colors (&normal, &to_mix, &real_bg, real_bg.alpha > 0., .25);
-		premix_colors (&hover, &to_mix, &real_bg, real_bg.alpha > 0., .35);
-		premix_colors (&active, &to_mix, &real_bg, real_bg.alpha > 0., .5);
-
-		normal_str = gdk_rgba_to_string (&normal);
-		hover_str = gdk_rgba_to_string (&hover);
-		active_str = gdk_rgba_to_string (&active);
-
 		g_string_append_printf (gstr,
 		                        "slider {"
-		                        " background-color: %s;"
+		                        " background-color: alpha(%s,.25);"
 		                        " transition-duration: 300ms;"
 		                        "}\n"
 		                        "slider:hover {"
-		                        " background-color: %s;"
+		                        " background-color: alpha(%s,.35);"
 		                        "}\n"
 		                        "slider.dragging:hover {"
-		                        " background-color: %s;"
+		                        " background-color: alpha(%s,.5);"
 		                        "}\n",
-					normal_str, hover_str, active_str);
-
-		g_free (normal_str);
-		g_free (hover_str);
-		g_free (active_str);
+		                        color, color, color);
 	}
 
 	g_free (background);
@@ -509,7 +480,8 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 
 	if (gstr->len > 0)
 	{
-		gtk_css_provider_load_from_string (priv->css_provider, gstr->str);
+		/* use -1 to avoid a copy of the string */
+		gtk_css_provider_load_from_data (priv->css_provider, gstr->str, -1);
 	}
 
 	g_string_free (gstr, TRUE);
@@ -553,7 +525,7 @@ update_child_vadjustment (GtkSourceMap *map)
 
 	gtk_adjustment_set_value (child_vadj, new_value);
 
-	gtk_source_map_allocate_slider (map);
+	gtk_widget_queue_allocate (GTK_WIDGET (map));
 }
 
 static gboolean
@@ -745,11 +717,6 @@ gtk_source_map_measure (GtkWidget      *widget,
 
 	priv = gtk_source_map_get_instance_private (map);
 
-	if (priv->view == NULL)
-	{
-		return;
-	}
-
 	if (orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
 		if (priv->font_desc == NULL)
@@ -1275,12 +1242,20 @@ gtk_source_map_size_allocate (GtkWidget *widget,
                               int        baseline)
 {
 	GtkSourceMap *map = (GtkSourceMap *)widget;
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+	GdkRectangle area;
+	int min, nat;
 
 	g_assert (GTK_SOURCE_IS_MAP (map));
 
 	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->size_allocate (widget, width, height, baseline);
 
-	gtk_source_map_allocate_slider (map);
+	get_slider_position (map, width, height, &area);
+	gtk_widget_measure (GTK_WIDGET (priv->slider),
+	                    GTK_ORIENTATION_VERTICAL,
+			    width, &min, &nat, NULL, NULL);
+	area.height = MAX (nat, area.height);
+	gtk_widget_size_allocate (GTK_WIDGET (priv->slider), &area, -1);
 }
 
 static void
@@ -1293,27 +1268,19 @@ gtk_source_map_snapshot (GtkWidget   *widget,
 	g_assert (GTK_IS_WIDGET (widget));
 	g_assert (GTK_IS_SNAPSHOT (snapshot));
 
-	/* Render the slider behind the contents so they are more legible and
-	 * we can avoid an RGBA blend on top of the contents.
-	 */
-	gtk_widget_snapshot_child (GTK_WIDGET (map), GTK_WIDGET (priv->slider), snapshot);
-
 	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->snapshot (widget, snapshot);
+
+	gtk_widget_snapshot_child (GTK_WIDGET (map), GTK_WIDGET (priv->slider), snapshot);
 }
 
 static void
 gtk_source_map_constructed (GObject *object)
 {
-	PangoFontMap *font_map;
-
 	G_OBJECT_CLASS (gtk_source_map_parent_class)->constructed (object);
 
-	font_map = _gtk_source_utils_get_builder_blocks ();
-
-	if (font_map != NULL)
-	{
-		gtk_widget_set_font_map (GTK_WIDGET (object), font_map);
-	}
+#if ENABLE_FONT_CONFIG
+	load_override_font (GTK_SOURCE_MAP (object));
+#endif
 }
 
 static void
@@ -1426,7 +1393,6 @@ gtk_source_map_init (GtkSourceMap *map)
 	gtk_widget_add_controller (GTK_WIDGET (map), g_steal_pointer (&scroll));
 
 	press = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
-	gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (press), 0);
 	g_signal_connect_swapped (press,
 	                          "pressed",
 	                          G_CALLBACK (gtk_source_map_click_pressed),
