@@ -21,6 +21,12 @@
 
 #include "config.h"
 
+#if ENABLE_FONT_CONFIG
+# include <fontconfig/fontconfig.h>
+# include <pango/pangocairo.h>
+# include <pango/pangofc-fontmap.h>
+#endif
+
 #include "gtksourcebuffer.h"
 #include "gtksourcelanguage.h"
 #include "gtksourcelanguagemanager.h"
@@ -28,11 +34,10 @@
 #include "gtksourcestylescheme.h"
 #include "gtksourcestyleschemepreview.h"
 #include "gtksourceview.h"
-#include "gtksourceutils-private.h"
 
 /**
  * GtkSourceStyleSchemePreview:
- *
+ * 
  * A preview widget for [class@StyleScheme].
  *
  * This widget provides a convenient [class@Gtk.Widget] to preview a [class@StyleScheme].
@@ -77,30 +82,50 @@ static GParamSpec *properties [N_PROPS];
 static guint signals [N_SIGNALS];
 
 static GtkCssProvider *css_provider;
+#if ENABLE_FONT_CONFIG
+static FcConfig *map_font_config;
+#endif
 
 static void
-load_override_font (GtkSourceView *view,
-                    PangoFontMap  *font_map)
+load_override_font (GtkSourceView *view)
 {
 	static gsize initialized;
+	PangoFontMap *font_map;
 
 	if (g_once_init_enter (&initialized))
 	{
+#if ENABLE_FONT_CONFIG
+		const gchar *font_path = PACKAGE_DATADIR"/fonts/BuilderBlocks.ttf";
+		FcConfig *config = FcInitLoadConfigAndFonts ();
+		if (!g_file_test (font_path, G_FILE_TEST_IS_REGULAR))
+			g_debug ("\"%s\" is missing or inaccessible", font_path);
+		FcConfigAppFontAddFile (config, (const FcChar8 *)font_path);
+		map_font_config = config;
+#endif
+
 		css_provider = gtk_css_provider_new ();
-		gtk_css_provider_load_from_string (css_provider,
-		                                   "textview, textview text { font-family: BuilderBlocks; font-size: 4px; line-height: 8px; }\n"
-		                                   "textview border.left gutter { padding-left: 2px; }\n");
+		gtk_css_provider_load_from_data (css_provider,
+		                                 "textview, textview text { font-family: BuilderBlocks; font-size: 4px; line-height: 8px; }\n"
+		                                 "textview border.left gutter { padding-left: 2px; }\n",
+		                                 -1);
 
 		g_once_init_leave (&initialized, TRUE);
 	}
 
-	_gtk_source_widget_add_css_provider (GTK_WIDGET (view), css_provider,
-	                                     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION-1);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (view)),
+	                                GTK_STYLE_PROVIDER (css_provider),
+	                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION-1);
 
-	if (font_map != NULL)
-	{
-		gtk_widget_set_font_map (GTK_WIDGET (view), font_map);
-	}
+#if ENABLE_FONT_CONFIG
+	font_map = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
+	pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (font_map), map_font_config);
+	gtk_widget_set_font_map (GTK_WIDGET (view), font_map);
+
+	g_assert (map_font_config != NULL);
+	g_assert (font_map != NULL);
+
+	g_object_unref (font_map);
+#endif
 }
 
 static void
@@ -234,7 +259,6 @@ gtk_source_style_scheme_preview_constructed (GObject *object)
 	GtkSourceStyleSchemePreview *self = (GtkSourceStyleSchemePreview *)object;
 	GtkSourceBuffer *buffer;
 	GtkSourceView *view;
-	PangoFontMap *font_map;
 	const char *name;
 	GtkLabel *label;
 
@@ -278,13 +302,7 @@ gtk_source_style_scheme_preview_constructed (GObject *object)
 
 	gtk_source_buffer_set_style_scheme (buffer, self->scheme);
 	add_text (buffer, self->scheme);
-
-	font_map = _gtk_source_utils_get_builder_blocks ();
-
-	if (font_map != NULL)
-	{
-		load_override_font (view, font_map);
-	}
+	load_override_font (view);
 
 	gtk_widget_set_parent (GTK_WIDGET (view), GTK_WIDGET (self));
 	gtk_widget_set_parent (GTK_WIDGET (label), GTK_WIDGET (self));
@@ -493,12 +511,12 @@ gtk_source_style_scheme_preview_set_selected (GtkSourceStyleSchemePreview *self,
 		if (selected)
 		{
 			gtk_widget_add_css_class (GTK_WIDGET (self), "selected");
-			gtk_widget_set_visible (GTK_WIDGET (self->image), TRUE);
+			gtk_widget_show (GTK_WIDGET (self->image));
 		}
 		else
 		{
 			gtk_widget_remove_css_class (GTK_WIDGET (self), "selected");
-			gtk_widget_set_visible (GTK_WIDGET (self->image), FALSE);
+			gtk_widget_hide (GTK_WIDGET (self->image));
 		}
 
 		g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SELECTED]);
